@@ -87,7 +87,7 @@ QString BBS::getMessage()
     message = (m_pBuffer->screen(0)->getText().simplified());
     line = m_pBuffer->screen(i);
     while (isUnicolor(line)) {
-        message += "\n" + (line->getText());
+        message += "\n" + (line->getText().simplified());
         i++;
         line = m_pBuffer->screen(i);
     }
@@ -131,8 +131,12 @@ void BBS::setPageState()
             m_nPageState = 0; //menu
     } else {
         line = m_pBuffer->screen(m_pBuffer->line() - 1);  // last line
-        if (isUnicolor(line))
-            m_nPageState = 2; // reading
+        if (isUnicolor(line)) {
+            if (line->getText().indexOf("Ctrl-Q")!=-1 || line->getText().indexOf("^X/^Q")!=-1)
+                m_nPageState = 4; // editing
+            else
+                m_nPageState = 2; // reading
+        }
     }
 }
 
@@ -143,7 +147,7 @@ int BBS::getCursorType(const QPoint& pt)
 
     QRect rc = getSelectRect();
 
-    int nCursorType = 8;
+    int nCursorType = 9;
 #ifdef SCRIPT_ENABLED
     if (m_scriptEngine != NULL) {
         m_scriptHelper->setAccepted(false);
@@ -164,7 +168,7 @@ int BBS::getCursorType(const QPoint& pt)
 
     switch (m_nPageState) {
     case -1: // not recognized
-        nCursorType = 8;
+        nCursorType = 9;
         break;
     case 0:  // menu
         if (pt.x() < 5) // LEFT
@@ -172,7 +176,7 @@ int BBS::getCursorType(const QPoint& pt)
         else if (rc.contains(pt))// HAND
             nCursorType = 7;
         else
-            nCursorType = 8;
+            nCursorType = 9;
         break;
     case 1:  // list
         if (pt.x() < 12) // LEFT
@@ -190,7 +194,7 @@ int BBS::getCursorType(const QPoint& pt)
         else if (rc.contains(pt))
             nCursorType = 7;
         else
-            nCursorType = 8;
+            nCursorType = 9;
         break;
     case 2:  // read
         if (pt.x() < 12) // LEFT
@@ -202,10 +206,12 @@ int BBS::getCursorType(const QPoint& pt)
                  && (pt.y() - m_nScreenStart) > m_pBuffer->line() / 2) // PAGEDOWN
             nCursorType = 3;
         else
-            nCursorType = 8;
+            nCursorType = 9;
         break;
     case 3:
         break;
+    case 4:
+        nCursorType = 8;
     default:
         break;
     }
@@ -252,7 +258,10 @@ void BBS::updateSelectRect()
 {
     QRect rect(0, 0, 0, 0);
 
-    m_pBuffer->at(m_rcSelection.y())->setChanged(-1,-1);
+	TextLine *line = m_pBuffer->at(m_rcSelection.y());
+	if (line == NULL)
+		return;
+	line->setChanged(-1,-1);
     // current screen scrolled
     if (m_nScreenStart != (m_pBuffer->lines() - m_pBuffer->line())) {
         m_rcSelection = rect;
@@ -286,8 +295,6 @@ void BBS::updateSelectRect()
     }
 #endif
 
-    TextLine * line = NULL;
-
     switch (m_nPageState) {
     case -1:
         break;
@@ -295,10 +302,12 @@ void BBS::updateSelectRect()
         if (m_ptCursor.y() - m_nScreenStart >= 7 &&
                 m_ptCursor.x() > 5) {
             line = m_pBuffer->at(m_ptCursor.y());
-            QString cstr = line->getText(0, m_ptCursor.x());
+			if (line == NULL)
+				break;
+			QString cstr = line->getText();
 
             QRegExp reg("[a-zA-Z0-9][).\\]]");
-            int indexChar = cstr.indexOf(reg);
+            int indexChar = cstr.lastIndexOf(reg, m_ptCursor.x());
             if (indexChar != -1) {
                 m_cMenuChar = cstr.at(indexChar).toLatin1();
 
@@ -306,7 +315,6 @@ void BBS::updateSelectRect()
                 if (indexChar > 0 && (cstr[indexChar-1] == '(' || cstr[indexChar-1] == '['))
                     nMenuStart--;
 
-                cstr = line->getText();
                 reg = QRegExp("[^ ]");
 
                 int nMenuBaseLength = 20;
@@ -319,7 +327,7 @@ void BBS::updateSelectRect()
                 if (nMenuEnd != -1) {
                     nMenuLength = line->beginIndex(nMenuEnd) - line->beginIndex(nMenuStart) + line->size(line->beginIndex(nMenuEnd));
                 }
-                if (m_ptCursor.x() >= nMenuStart && m_ptCursor.x() <= nMenuStart + nMenuLength) {
+                if (m_ptCursor.x() >= line->beginIndex(nMenuStart) && m_ptCursor.x() <= line->beginIndex(nMenuStart) + nMenuLength) {
                     rect.setX(line->beginIndex(nMenuStart));
                     rect.setY(m_ptCursor.y());
                     rect.setWidth(nMenuLength);
@@ -334,6 +342,8 @@ void BBS::updateSelectRect()
                 m_ptCursor.x() >= 12 && m_ptCursor.x() <= m_pBuffer->columns() - 16) {
 
             line =  m_pBuffer->at(m_ptCursor.y());
+			if (line == NULL)
+				break;
             QString str = line->getText();
             if (str.count(" ") != (int) str.length()) {
                 rect.setX(0);
@@ -424,7 +434,7 @@ bool BBS::checkUrl(QRect & rcUrl, QRect & rcOld)
             if (pt > url.first && pt < url.second) {
                 //qDebug() << "get text: " << getText(url.first, url.second);
                 m_strUrl = getText(url.first, url.second);
-                QRegExp urlRe("^(mailto:|(https?|mms|rstp|ftp|gopher|telnet|ed2k|file)://)");
+                QRegExp urlRe("(mailto:|(https?|mms|rstp|ftp|gopher|telnet|ed2k|file)://)");
                 QRegExp emailRe("^[A-Z0-9._%-]+@[A-Z0-9.-]+\\.[A-Z]{2,4}");
                 if (urlRe.indexIn(m_strUrl)==-1) {
                     if (emailRe.indexIn(m_strUrl)==-1) {
@@ -542,6 +552,9 @@ void BBS::updateUrlList()
                     }
                     multiline = true;
                 }
+                if (index2 == -1) {
+                    index2 = m_pBuffer->screen(i)->getText().length() - 1;
+                }
                 int urlEnd = i*m_pBuffer->columns() + m_pBuffer->screen(i)->beginIndex(index2);
                 if (verifyUrl(urlBegin, urlEnd))
                     m_urlPosList.append(qMakePair(urlBegin, urlEnd));
@@ -566,6 +579,10 @@ bool BBS::verifyUrl(int urlBegin, int urlEnd)
     int ip_begin = 0;
     int ip_end = 0;
 
+    if (urlEnd < urlBegin) {
+        return false;
+    }
+
     QString strText = getText(urlBegin, urlEnd);
 
     int nNoType = -1;
@@ -578,7 +595,7 @@ bool BBS::verifyUrl(int urlBegin, int urlEnd)
             if ((ata = strText.indexOf('@', begin + 1)) == -1)
                 host = url + (ata - begin) + 1;
             else
-                return -1;
+                return false;
         } else {
             host = url+urlRe.matchedLength();
         }
@@ -658,7 +675,7 @@ int BBS::checkUrlBegin( const QString & lineText, int index)
 {
     int i = 0;
     QString urlText = lineText;
-    for (i = index; i < urlText.length() && isIllChar(urlText.at(i)); i++);
+    for (i = index; i < urlText.length() && isIllChar(urlText.at(i)); i++) ;
     if (i < urlText.length()) {
         return i;
     }
@@ -672,7 +689,7 @@ int BBS::checkUrlEnd( const QString & lineText, int index)
     if (urlText.isEmpty()) {
         return 0;
     }
-    for (i = index; i< urlText.length() && !isIllChar(urlText.at(i)); i++);
+    for (i = index; i< urlText.length() && !isIllChar(urlText.at(i)); i++) ;
     if (i < urlText.length()) {
         return i;
     }
