@@ -13,7 +13,7 @@
 #define TERMSTRING_H
 
 #include <QtCore/QString>
-#include <QByteArray>
+#include <QList>
 
 namespace QTerm
 {
@@ -31,8 +31,10 @@ public:
     QString mid(int index, int len);
     QString string();
     int beginIndex(int pos);
+    int pos(int index);
     int size(int index);
     bool isPartial(int index);
+    bool isEmpty();
     static int wcwidth(QChar ch);
 private:
     struct interval {
@@ -41,8 +43,10 @@ private:
     };
     static int bisearch(QChar ucs, const struct interval *table, int max);
     void updateIndex();
+    void dumpIndex();
     QString m_string;
-    QByteArray m_index;
+    //TODO: Switch to QVector after we drop Qt4 support
+    QList<int> m_index;
 };
 } // namespace QTerm
 
